@@ -21,20 +21,24 @@
 #define _EXCEPTION_H_
 
 #include <exception>
+#include "String++.h"
 
 /* Generic excpetion class for throwing exceptions */
-class Exception : public exception {
+class Exception : public std::exception {
 public:
   Exception (char const *where, char const *message, int appErrNo = 0);
-  char const *what() const;
+  Exception (char const *where, const String &message, int appErrNo = 0);
+  ~Exception () throw () {}
+  char const *what() const throw();
   int errNo() const;
 private:
-  char const *_message;
+  String _message;
   int appErrNo;
 };
 
 // Where should these live?
 #define APPERR_CORRUPT_PACKAGE	1
 #define APPERR_IO_ERROR		2
+#define APPERR_LOGIC_ERROR	3
 
 #endif /* _EXCEPTION_H_ */
