@@ -26,7 +26,7 @@ class IniParseFeedback;
 class IniParseFindVisitor : public FindVisitor
 {
 public:
-  IniParseFindVisitor (IniDBBuilder &aBuilder, String const &localroot, IniParseFeedback const &);
+  IniParseFindVisitor (IniDBBuilder &aBuilder, String const &localroot, IniParseFeedback &);
   virtual void visitFile(String const &basePath, const WIN32_FIND_DATA *);
   virtual ~ IniParseFindVisitor ();
 
@@ -38,7 +38,7 @@ protected:
   IniParseFindVisitor & operator= (IniParseFindVisitor const &);
 private:
   IniDBBuilder &_Builder;
-  IniParseFeedback const &_feedback;
+  IniParseFeedback &_feedback;
   unsigned int baseLength;
   int local_ini;
   char *error_buf;
