@@ -14,13 +14,16 @@
  */
 
 #include "LogSingleton.h"
+#include <sstream>
 
 // Logging class. Default logging level is PLAIN.
 class LogFile : public LogSingleton {
 public:
+  static LogFile *createLogFile();
   LogFile();
   void clearFiles(); // delete all target filenames
   void setFile (int minlevel, String const &path, bool append);
+  String getFileName (int level) const;
   /* Some platforms don't call destructors. So this call exists
    * which guarantees to flush any log data...
    * but doesn't call generic C++ destructors
@@ -28,9 +31,10 @@ public:
   virtual void exit (int const exit_code) __attribute__ ((noreturn));
   virtual ~LogFile();
   // get a specific verbosity stream.
-  virtual ostream &operator() (enum log_level level);
+  virtual std::ostream &operator() (enum log_level level);
   
 protected:
+  LogFile(std::stringbuf *aStream);
   LogFile (LogFile const &); // no copy constructor
   LogFile &operator = (LogFile const&); // no assignment operator
   virtual void endEntry(); // the current in-progress entry is complete.
