@@ -27,7 +27,7 @@
 
 #if 0
 static const char *cvsid =
-  "\n%%% $Id: choose.cc,v 2.99 2002/05/19 06:23:37 rbcollins Exp $\n";
+  "\n%%% $Id: choose.cc,v 2.113 2003/03/13 21:21:14 rbcollins Exp $\n";
 #endif
 
 #include "win32.h"
@@ -41,7 +41,7 @@ static const char *cvsid =
 #include "resource.h"
 #include "state.h"
 #include "msg.h"
-#include "log.h"
+#include "LogSingleton.h"
 #include "filemanip.h"
 #include "io_stream.h"
 #include "propsheet.h"
@@ -51,7 +51,6 @@ static const char *cvsid =
 #include "package_db.h"
 #include "package_meta.h"
 #include "package_version.h"
-#include "cygpackage.h"
 
 #include "PickView.h"
 
@@ -59,7 +58,9 @@ static const char *cvsid =
 #include "threebar.h"
 
 #include "download.h"
-  
+
+using namespace std;
+
 extern ThreeBarProgressPage Progress;
 
 static int initialized = 0;
@@ -150,7 +151,7 @@ list_click (HWND hwnd, BOOL dblclk, int x, int y, UINT hitCode)
       SCROLLINFO si;
       memset (&si, 0, sizeof (si));
       si.cbSize = sizeof (si);
-      si.fMask = SIF_ALL;	/* SIF_RANGE was giving strange behaviour */
+      si.fMask = SIF_ALL | SIF_DISABLENOSCROLL;	/* SIF_RANGE was giving strange behaviour */
       si.nMin = 0;
 
       si.nMax = chooser->contents.itemcount () * chooser->row_height;
@@ -217,7 +218,7 @@ listview_proc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 		GetClientRect (hwnd, &r);
 		SCROLLINFO si;
 		si.cbSize = sizeof (si);
-		si.fMask = SIF_ALL;
+		si.fMask = SIF_ALL | SIF_DISABLENOSCROLL;
 		GetScrollInfo (hwnd, SB_HORZ, &si);
 		int oldMax = si.nMax;
 		si.nMax =
@@ -269,43 +270,52 @@ static void
 set_existence ()
 {
   packagedb db;
+  /* binary packages */
   /* Remove packages that are in the db, not installed, and have no 
-     mirror info. */
-  size_t n = 1;
-  while (n <= db.packages.number ())
+     mirror info and are not cached for both binary and source packages. */
+  vector <packagemeta *>::iterator i = db.packages.begin ();
+  while (i != db.packages.end ())
     {
-      packagemeta & pkg = *db.packages[n];
-      bool mirrors = false;
-      size_t o = 1;
-      while (o <= pkg.versions.number () && !mirrors)
+      packagemeta & pkg = **i;
+      if (!pkg.installed && !pkg.accessible() && 
+	  !pkg.sourceAccessible() )
 	{
-	  packageversion & ver = *pkg.versions[o];
-	  if (((source != IDC_SOURCE_CWD) && (ver.bin.sites.number ()
-					      || ver.src.sites.number ()))
-	      || ver.bin.Cached () || ver.src.Cached ())
-	    mirrors = true;
-	  ++o;
-	}
-      if (!pkg.installed && !mirrors)
-	{
-	  packagemeta *pkgm = db.packages.removebyindex (n);
+	  packagemeta *pkgm = *i;
 	  delete pkgm;
+	  i = db.packages.erase (i);
 	}
       else
-	++n;
+	++i;
     }
+#if 0
+  /* remove any source packages which are not accessible */
+  vector <packagemeta *>::iterator i = db.sourcePackages.begin();
+  while (i != db.sourcePackages.end())
+    {
+      packagemeta & pkg = **i;
+      if (!packageAccessible (pkg))
+	{
+	  packagemeta *pkgm = *i;
+	  delete pkgm;
+	  i = db.sourcePackages.erase (i);
+	}
+      else
+	++i;
+    }
+#endif
 }
 
 static void
 fill_missing_category ()
 {
   packagedb db;
-  for (size_t n = 1; n <= db.packages.number (); n++)
+  for (vector <packagemeta *>::iterator i = db.packages.begin ();
+       i != db.packages.end (); ++i)
     {
-      packagemeta & pkg = *db.packages[n];
-      if (!pkg.Categories.number ())
-	pkg.add_category (db.categories.registerbykey ("Misc"));
-      pkg.add_category (db.categories.registerbykey ("All"));
+      packagemeta & pkg = **i;
+      if (!pkg.categories.size ())
+	pkg.add_category ("Misc");
+      pkg.add_category ("All");
     }
 }
 
@@ -314,38 +324,33 @@ default_trust (HWND h, trusts trust)
 {
   chooser->deftrust = trust;
   packagedb db;
-  for (size_t n = 1; n <= db.packages.number (); n++)
+  for (vector <packagemeta *>::iterator i = db.packages.begin ();
+       i != db.packages.end (); ++i)
     {
-      packagemeta & pkg = *db.packages[n];
+      packagemeta & pkg = **i;
       if (pkg.installed
-	  || pkg.Categories.getbykey (db.categories.registerbykey ("Base"))
-	  || pkg.Categories.getbykey (db.categories.registerbykey ("Misc")))
+	  || pkg.categories.find ("Base") != pkg.categories.end ()
+	  || pkg.categories.find ("Misc") != pkg.categories.end ())
 	{
 	  pkg.desired = pkg.trustp (trust);
 	  if (pkg.desired)
-	    {
-	      pkg.desired->binpicked = pkg.desired == pkg.installed ? 0 : 1;
-	      pkg.desired->srcpicked = 0;
-	    }
+	    pkg.desired.pick (pkg.desired.accessible() 
+	      		      && pkg.desired != pkg.installed);
 	}
       else
-	pkg.desired = 0;
+	pkg.desired = packageversion ();
     }
   RECT r;
   GetClientRect (h, &r);
   InvalidateRect (h, &r, TRUE);
   // and then do the same for categories with no packages.
-  size_t n = 1;
-  while (n <= db.categories.number ())
-    {
-      if (!db.categories[n]->packages)
-	{
-	  Category *cat = db.categories.removebyindex (n);
-	  delete cat;
-	}
-      else
-	++n;
-    }
+  for (packagedb::categoriesType::iterator n = packagedb::categories.begin();
+       n != packagedb::categories.end(); ++n)
+    if (!n->second.size())
+      {
+	log (LOG_BABBLE) << "Removing empty category " << n->first << endLog;
+        packagedb::categories.erase (n++);
+      }
 }
 
 static void
@@ -357,28 +362,50 @@ set_view_mode (HWND h, PickView::views mode)
   packagedb db;
   if (chooser->get_view_mode () == PickView::views::Package)
     {
-      for (size_t n = 1; n <= db.packages.number (); n++)
+      for (vector <packagemeta *>::iterator i = db.packages.begin ();
+	   i != db.packages.end (); ++i)
 	{
-	  packagemeta & pkg = *db.packages[n];
+	  packagemeta & pkg = **i;
 	  if ((!pkg.desired && pkg.installed)
-	      || (pkg.desired
-		  && (pkg.desired->srcpicked || pkg.desired->binpicked)))
+	      || (pkg.desired && (pkg.desired.picked () 
+		  || pkg.desired.sourcePackage().picked())))
+	    chooser->insert_pkg (pkg);
+	}
+    }
+  else if (chooser->get_view_mode () == PickView::views::PackageKeeps)
+    {
+      for (vector <packagemeta *>::iterator i = db.packages.begin ();
+	   i != db.packages.end (); ++i)
+	{
+	  packagemeta & pkg = **i;
+	  if (pkg.installed && pkg.desired && !pkg.desired.picked() 
+	      && !pkg.desired.sourcePackage().picked())
+	    chooser->insert_pkg (pkg);
+	}
+    }
+  else if (chooser->get_view_mode () == PickView::views::PackageSkips)
+    {
+      for (vector <packagemeta *>::iterator i = db.packages.begin ();
+	   i != db.packages.end (); ++i)
+	{
+	  packagemeta & pkg = **i;
+	  if (!pkg.desired && !pkg.installed)
 	    chooser->insert_pkg (pkg);
 	}
     }
   else if (chooser->get_view_mode () == PickView::views::PackageFull)
     {
-      for (size_t n = 1; n <= db.packages.number (); n++)
-	{
-	  packagemeta & pkg = *db.packages[n];
-	  chooser->insert_pkg (pkg);
-	}
+      for (vector <packagemeta *>::iterator i = db.packages.begin ();
+	   i != db.packages.end (); ++i)
+	chooser->insert_pkg (**i);
     }
   else if (chooser->get_view_mode () == PickView::views::Category)
     {
       /* start collapsed. TODO: make this a chooser flag */
-      for (size_t n = 1; n <= db.categories.number (); n++)
-	chooser->insert_category (db.categories[n], CATEGORY_COLLAPSED);
+      for (packagedb::categoriesType::iterator n 
+	   = packagedb::categories.begin();
+	   n != packagedb::categories.end(); ++n)
+	chooser->insert_category (&*n, CATEGORY_COLLAPSED);
     }
 
   RECT r;
@@ -386,7 +413,7 @@ set_view_mode (HWND h, PickView::views mode)
   SCROLLINFO si;
   memset (&si, 0, sizeof (si));
   si.cbSize = sizeof (si);
-  si.fMask = SIF_ALL;
+  si.fMask = SIF_ALL | SIF_DISABLENOSCROLL;
   si.nMin = 0;
   si.nMax = chooser->headers[chooser->last_col].x + chooser->headers[chooser->last_col].width;	// + HMARGIN;
   si.nPage = r.right;
@@ -417,20 +444,21 @@ create_listview (HWND dlg, RECT * r)
   packagedb db;
   chooser =
     new PickView (PickView::views::Category, lv,
-		  db.categories.registerbykey ("All"));
+		  *db.categories.find("All"));
 
   default_trust (lv, TRUST_CURR);
   set_view_mode (lv, PickView::views::Category);
   if (!SetDlgItemText (dlg, IDC_CHOOSE_VIEWCAPTION, chooser->mode_caption ()))
-    log (LOG_BABBLE, "Failed to set View button caption %ld",
-	 GetLastError ());
-  for (size_t n = 1; n <= db.packages.number (); n++)
+    log (LOG_BABBLE) << "Failed to set View button caption %ld" << 
+	 GetLastError () << endLog;
+  for (vector <packagemeta *>::iterator i = db.packages.begin ();
+       i != db.packages.end (); ++i)
     {
-      packagemeta & pkg = *db.packages[n];
+      packagemeta & pkg = **i;
       pkg.set_requirements (chooser->deftrust);
     }
   /* FIXME: do we need to init the desired fields ? */
-  static int ta[] = { IDC_CHOOSE_PREV, IDC_CHOOSE_CURR, IDC_CHOOSE_EXP, 0 };
+  static int ta[] = { IDC_CHOOSE_KEEP, IDC_CHOOSE_PREV, IDC_CHOOSE_CURR, IDC_CHOOSE_EXP, 0 };
   rbset (dlg, ta, IDC_CHOOSE_CURR);
 }
 
@@ -451,6 +479,19 @@ GetParentRect (HWND parent, HWND child, RECT * r)
   r->bottom = p.y;
 }
 
+static void
+scanAVersion (packageversion version)
+{
+  if (!version)
+    return;
+  /* Remove mirror sites.
+   * FIXME: This is a bit of a hack. a better way is to abstract
+   * the availability logic to the package
+   */
+  if (!check_for_cached (*(version.source())) && source == IDC_SOURCE_CWD)
+    version.source()->sites.clear();
+}
+
 static void
 scan_downloaded_files ()
 {
@@ -458,45 +499,37 @@ scan_downloaded_files ()
    * and fill in the Cached attribute if it exists.
    */
   packagedb db;
-  for (size_t n = 1; n <= db.packages.number (); ++n)
+  for (vector <packagemeta *>::iterator n = db.packages.begin ();
+       n != db.packages.end (); ++n)
     {
-      packagemeta & pkg = *db.packages[n];
-      for (size_t m = 1; m <= pkg.versions.number (); ++m)
+      packagemeta & pkg = **n;
+      for (set<packageversion>::iterator i = pkg.versions.begin (); 
+	   i != pkg.versions.end (); ++i)
 	{
-	  packageversion *version = pkg.versions[m];
-	  /* Remove mirror sites.
-	   * FIXME: This is a bit of a hack. a better way is to abstract
-	   * the availability logic to the package
-	   */
-	  if (!check_for_cached (version->bin) && source == IDC_SOURCE_CWD)
-	    while (version->bin.sites.number())
-	      {
-		site *asite = version->bin.sites.removebyindex(1);
-		delete asite;
-	      }
-	  if (!check_for_cached (version->src) && source == IDC_SOURCE_CWD)
-	    while (version->src.sites.number())
-	      {
-		site *asite = version->src.sites.removebyindex(1);
-		delete asite;
-	      }
+	  scanAVersion (*i);
+	  packageversion foo = *i;
+	  packageversion pkgsrcver = foo.sourcePackage();
+	  scanAVersion (pkgsrcver);
 	  /* For local installs, if there is no src and no bin, the version
 	   * is unavailable
 	   */
-	  if (!version->src.Cached() && !version->bin.Cached() 
-	      && source == IDC_SOURCE_CWD && version != pkg.installed)
+	  if (!i->accessible() && !pkgsrcver.accessible()
+	      && *i != pkg.installed)
 	    {
-	      pkg.versions.removebyindex(m--);
-	      if (pkg.prev == version)
-		pkg.prev = NULL;
-	      if (pkg.curr == version)
-		pkg.curr = NULL;
-	      if (pkg.exp == version)
-		pkg.exp = NULL;
-	      delete version;
+	      if (pkg.prev == *i)
+		pkg.prev = packageversion();
+	      if (pkg.curr == *i)
+		pkg.curr = packageversion();
+	      if (pkg.exp == *i)
+		pkg.exp = packageversion();
+	      pkg.versions.erase(i);
+	      /* For now, leave the source version alone */
 	    }
 	}
     }
+  /* Don't explicity iterate through sources - any sources that aren't 
+     referenced are unselectable anyway 
+     */
 }
 
 bool
@@ -531,6 +564,12 @@ ChooserPage::OnInit ()
   create_listview (GetHWND (), &r);
 }
 
+void
+ChooserPage::OnActivate()
+{
+    set_view_mode (lv, chooser->get_view_mode ());
+}
+
 long
 ChooserPage::OnNext ()
 {
@@ -545,64 +584,44 @@ ChooserPage::OnNext ()
       Progress.SetActivateTask (WM_APP_START_DOWNLOAD);
     }
 
-  log (LOG_BABBLE, "Chooser results...");
+  log (LOG_BABBLE) << "Chooser results..." << endLog;
   packagedb db;
-  for (size_t n = 1; n <= db.packages.number (); n++)
+  for (vector <packagemeta *>::iterator i = db.packages.begin ();
+       i != db.packages.end (); ++i)
     {
-      packagemeta & pkg = *db.packages[n];
-//      static const char *infos[] = { "nada", "prev", "curr", "test" };
+      packagemeta & pkg = **i;
       const char *trust = ((pkg.desired == pkg.prev) ? "prev"
 			   : (pkg.desired == pkg.curr) ? "curr"
 			   : (pkg.desired == pkg.exp) ? "test" : "unknown");
       String action = pkg.action_caption ();
       String const installed =
-	pkg.installed ? pkg.installed->Canonical_version () : "none";
+	pkg.installed ? pkg.installed.Canonical_version () : "none";
 
-      log (LOG_BABBLE, "[%s] action=%s trust=%s installed=%s"
-	   " src?=%s",
-	   pkg.name.cstr_oneuse (), action.cstr_oneuse (), trust,
-	   installed.cstr_oneuse (), pkg.desired
-	   && pkg.desired->srcpicked ? "yes" : "no");
-      if (pkg.Categories.number ())
+      log (LOG_BABBLE) << "[" << pkg.name << "] action=" << action << " trust=" << trust << " installed=" << installed << " src?=" << (pkg.desired && pkg.desired.sourcePackage().picked() ? "yes" : "no") << endLog;
+      if (pkg.categories.size ())
 	{
 	  /* List categories the package belongs to */
-	  String all_categories = pkg.Categories[1]->key.name;
-	  for (size_t n = 2; n <= pkg.Categories.number (); n++)
-	    all_categories += String (", ") + pkg.Categories[n]->key.name;
+	  set <String, String::caseless>::const_iterator i 
+	    = pkg.categories.begin ();
+	  String all_categories = *(i++);
+	  while (i != pkg.categories.end ())
+	    all_categories += String (", ") + *(i++);
 
-	  log (LOG_BABBLE, String ("     categories=") + all_categories);
+	  log (LOG_BABBLE) << "     categories=" << all_categories << endLog;
 	}
-      if (pkg.desired && pkg.desired->required)
+#if 0
+      if (pkg.desired.required())
 	{
 	  /* List other packages this package depends on */
 	  Dependency *dp = pkg.desired->required;
-	  String requires = dp->package;
+	  String requires = dp->package.serialise ();
 	  for (dp = dp->next; dp; dp = dp->next)
-	    if (dp->package.size ())
-	      requires += String (", ") + dp->package;
+	    requires += String (", ") + dp->package.serialise ();
 
-	  log (LOG_BABBLE, String ("     requires=") + requires);
-	}
-#if 0
-
-      /* FIXME: Reinstate this code, but spit out all mirror sites */
-
-      for (int t = 1; t < NTRUST; t++)
-	{
-	  if (pkg->info[t].install)
-	    log (LOG_BABBLE, "     [%s] ver=%s\n"
-		 "          inst=%s %d exists=%s\n"
-		 "          src=%s %d exists=%s",
-		 infos[t],
-		 pkg->info[t].version ? : "(none)",
-		 pkg->info[t].install ? : "(none)",
-		 pkg->info[t].install_size,
-		 (pkg->info[t].install_exists) ? "yes" : "no",
-		 pkg->info[t].source ? : "(none)",
-		 pkg->info[t].source_size,
-		 (pkg->info[t].source_exists) ? "yes" : "no");
+	  log (LOG_BABBLE) << "     requires=" << requires;
 	}
 #endif
+      pkg.logAllVersions();
     }
   return IDD_INSTATUS;
 }
@@ -629,13 +648,28 @@ ChooserPage::OnMessageCmd (int id, HWND hwndctl, UINT code)
   packagedb db;
   switch (id)
     {
+    case IDC_CHOOSE_KEEP:
+      if (IsDlgButtonChecked (GetHWND (), id))
+      {
+	for (vector <packagemeta *>::iterator i = db.packages.begin ();
+	     i != db.packages.end (); ++i)
+          {
+            packagemeta & pkg = **i;
+            pkg.desired = pkg.installed;
+          }
+        set_view_mode (lv, chooser->get_view_mode ());
+        break;
+      }
+      else
+      return false;
     case IDC_CHOOSE_PREV:
       if (IsDlgButtonChecked (GetHWND (), id))
       {
         default_trust (lv, TRUST_PREV);
-        for (size_t n = 1; n <= db.packages.number (); n++)
+	for (vector <packagemeta *>::iterator i = db.packages.begin ();
+	     i != db.packages.end (); ++i)
           {
-            packagemeta & pkg = *db.packages[n];
+            packagemeta & pkg = **i;
             pkg.set_requirements (TRUST_PREV);
           }
         set_view_mode (lv, chooser->get_view_mode ());
@@ -647,9 +681,10 @@ ChooserPage::OnMessageCmd (int id, HWND hwndctl, UINT code)
       if (IsDlgButtonChecked (GetHWND (), id))
       {
         default_trust (lv, TRUST_CURR);
-        for (size_t n = 1; n <= db.packages.number (); n++)
+	for (vector <packagemeta *>::iterator i = db.packages.begin ();
+	     i != db.packages.end (); ++i)
           {
-            packagemeta & pkg = *db.packages[n];
+            packagemeta & pkg = **i;
             pkg.set_requirements (TRUST_CURR);
           }
         set_view_mode (lv, chooser->get_view_mode ());
@@ -661,9 +696,10 @@ ChooserPage::OnMessageCmd (int id, HWND hwndctl, UINT code)
       if (IsDlgButtonChecked (GetHWND (), id))
       {
         default_trust (lv, TRUST_TEST);
-        for (size_t n = 1; n <= db.packages.number (); n++)
+	for (vector <packagemeta *>::iterator i = db.packages.begin ();
+	     i != db.packages.end (); ++i)
           {
-            packagemeta & pkg = *db.packages[n];
+            packagemeta & pkg = **i;
             pkg.set_requirements (TRUST_TEST);
           }
         set_view_mode (lv, chooser->get_view_mode ());
@@ -675,8 +711,8 @@ ChooserPage::OnMessageCmd (int id, HWND hwndctl, UINT code)
       set_view_mode (lv, ++chooser->get_view_mode ());
       if (!SetDlgItemText
         (GetHWND (), IDC_CHOOSE_VIEWCAPTION, chooser->mode_caption ()))
-      log (LOG_BABBLE, "Failed to set View button caption %ld",
-           GetLastError ());
+      log (LOG_BABBLE) << "Failed to set View button caption " << 
+           GetLastError () << endLog;
       break;
 
 
