@@ -39,6 +39,11 @@ public:
   virtual void OnInit ();
   virtual long OnNext ();
   virtual long OnBack ();
+  virtual void OnActivate ();
+  virtual long OnUnattended ()
+  {
+    return OnNext ();
+  }; 
 };
 
 #endif /* __cplusplus */
