@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2002 Robert Collins.
+ * Copyright (c) 2003 Robert Collins.
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -18,6 +19,7 @@
 
 #include <getopt.h>
 #include <iosfwd>
+#include <vector>
 
 class Option;
 
@@ -28,15 +30,15 @@ public:
   virtual ~OptionSet();
   virtual void Register (Option *);
   virtual bool Process (int argc, char **argv, OptionSet *defaultOptionSet=0);
-  virtual void ParameterUsage (ostream &);
+  virtual void ParameterUsage (std::ostream &);
+  virtual std::vector<Option *> const &optionsInSet() const;
 protected:
   OptionSet (OptionSet const &);
   OptionSet &operator= (OptionSet const &);
   // force initialisation of variables
   void Init ();
 private:
-  Option **options;
-  int optCount;
+  std::vector<Option *> options;
 };
 
 #endif // _OPTIONSET_H_
