@@ -17,17 +17,23 @@
 #define _PACKAGE_DB_H_
 
 /* required to parse this file */
-#include "list.h"
 #include <String++.h>
-class Category;
+#include <vector>
+#include <map>
+#include "category.h"
 class packagemeta;
 class io_stream;
+class PackageSpecification;
 
 typedef enum {
   PackageDB_Install,
   PackageDB_Download
 } PackageDBActions;
 
+
+class packagedb;
+typedef std::vector <packagemeta *>::iterator PackageDBConnectedIterator;
+
 /*TODO: add mutexs */
 class packagedb
 {
@@ -35,13 +41,22 @@ public:
   packagedb ();
   /* 0 on success */
   int flush ();
-  /* all seen packages */
-  static list < packagemeta, String, String::casecompare > packages;
+  packagemeta * findBinary (PackageSpecification const &) const;
+  packagemeta * findSource (PackageSpecification const &) const;
+  PackageDBConnectedIterator connectedBegin();
+  PackageDBConnectedIterator connectedEnd();
+  /* all seen binary packages */
+  static std::vector < packagemeta *> packages;
+  /* all seen source packages */
+  static std::vector <packagemeta *> sourcePackages;
   /* all seen categories */
-  static list < Category, String, String::casecompare > categories;
+  typedef std::map <String, std::vector <packagemeta *>, String::caseless > categoriesType;
+  static categoriesType categories;
   static PackageDBActions task;
 private:
   static int installeddbread;	/* do we have to reread this */
+  friend class ConnectedLoopFinder;
+  static std::vector <packagemeta *> dependencyOrderedPackages;
 };
 
 #endif /* _PACKAGE_DB_H_ */
