@@ -18,7 +18,7 @@
 
 #if 0
 static const char *cvsid =
-  "\n%%% $Id: postinstall.cc,v 2.9 2002/05/19 03:07:51 rbcollins Exp $\n";
+  "\n%%% $Id: postinstall.cc,v 2.11 2003/03/18 22:43:15 maxb Exp $\n";
 #endif
 
 #include "dialog.h"
@@ -26,6 +26,9 @@ static const char *cvsid =
 #include "mount.h"
 #include "script.h"
 #include "FindVisitor.h"
+#include "FilterVisitor.h"
+#include "package_db.h"
+#include "package_meta.h"
 
 class RunFindVisitor : public FindVisitor
 {
@@ -47,7 +50,19 @@ do_postinstall (HINSTANCE h, HWND owner)
   next_dialog = 0;
   init_run_script ();
   SetCurrentDirectory (get_root_dir ().cstr_oneuse());
+  packagedb db;
+  PackageDBConnectedIterator i = db.connectedBegin ();
+  while (i != db.connectedEnd ())
+    {
+      packagemeta & pkg = **i;
+      if (pkg.installed)
+	for (std::vector<Script>::iterator script=pkg.installed.scripts().begin(); script != pkg.installed.scripts().end(); ++script) 
+	  run_script ("/etc/postinstall/", script->baseName());
+      ++i;
+    }
   RunFindVisitor myVisitor;
+  ExcludeNameFilter notDone("*.done");
+  FilterVisitor excludeDoneVisitor(&myVisitor, &notDone);
   String postinst = cygpath ("/etc/postinstall");
-  Find (postinst).accept (myVisitor);
+  Find (postinst).accept (excludeDoneVisitor);
 }
