@@ -33,6 +33,11 @@ public:
 
   bool Create ();
   virtual void OnInit ();
+  virtual long OnNext ();
+  long OnUnattended ()
+  {
+    return 0;
+  };
 };
 
 #endif // CINSTALL_SPLASH_H
